@@ -1,6 +1,6 @@
 #include <errno.h>
-#include <iostream.h>
-#include <fstream.h>
+#include <iostream>
+#include <fstream>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -15,6 +15,8 @@
 #include "calcdb.h"
 #include "Array.h"
 
+using namespace std;
+
 #define MEG (1024*1204)
 struct timeval tp;
 #define seconds(tm) gettimeofday(&tp,(struct timezone *)0);\
@@ -58,7 +60,7 @@ void parse_args(int argc, char **argv)
    int c;
    
    if (argc < 2) {
-      cout << "usage: exttpose [OPTION]... -i<infile> -o<outfile>\n";
+       cout << "usage: exttpose [OPTION]... -i<infile> -o<outfile>\n";
       exit(EXIT_FAILURE);
    }
    else{
